//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : Matrix
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Objects constructed from the Matrix class act as 2-dimensional
// indexed collections. These structures are always rectangular, not ragged.
// Not only should you study how to use this class, but you should also
// study the array of arrays that is used to implement it.
//
// Known Bugs   : None
//
// Future Plans : None
//
// Program History:
//   5/18/01: R. Pattis - Operational for 15-100
//   8/20/01: R. Pattis - JavaDoc comments added
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;





/** 
 * Objects constructed from the <code>Matrix</code> class act
 *   as 2-dimensional indexed collections.
 * These structures are always rectangular, not ragged.
 * Not only should you study how to use this class, but you should also
 *   study the array of arrays that is used to implement it. * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
*/
public class Matrix { 


	/** 
	 * Constructs a <code>Natrix</code> object, specifying the number
	 *   rows and columns in the matrix.
	 *
	 * @param  rowSize specifies the number of rows
	 * @param  colSize specifies the number of rows
	 *
	 * @throws IllegalArgumentException if either parameter is less than 1
	*/
  public Matrix (int rowSize, int colSize) throws IllegalArgumentException
  {
    if (rowSize < 1)
      throw new IllegalArgumentException("Matrix constructor: rowSize ("+rowSize+") < 1"); 
    if (colSize < 1)
      throw new IllegalArgumentException("Matrix constructor: colSize ("+colSize+") < 1"); 

    values = new Object[rowSize][colSize];
 
    //Each row has the same number of columns.
    //All are set to null automatically
    //for (int r=0; r<values.length; r++)
    //  values[r] = new Object[colSize];
  }
  
  
  
	/** 
	 * Constructs a <code>Matrix</code> object, that copies <code>other</b>.
	 *
	 * @param  other specifies the <code>Matrix</code> to copy for initialization
	*/
  public Matrix (Matrix other)
  {
    this(other.getRowSize(), other.getColSize());
    for (int r=0; r<other.getRowSize(); r++)
      for (int c=0; c<other.getColSize(); c++)
         values[r][c] = other.values[r][c];
  }
  
  
  
	/** 
	 * Returns the number of rows in this Matrix.
	 *
	 * @return the number of rows in this Matrix
	*/
  public int getRowSize ()
  {return values.length;}
  
  
  
    
	/** 
	 * Returns the number of columns in this Matrix.
	 *
	 * @return the number of columns in this Matrix
	*/
  public int getColSize ()
  {return values[0].length;}  //Each row has the same number of columns
  
  
    

  
  
	/** 
	 * Returns the number of references to objects storable in this Matrix.
	 *
	 * @return the number of references to objects storable in this Matrix
	*/
  public int getSize ()
  {return getRowSize()*getColSize();}
  
  
    

  
  
	/** 
	 * Returns a reference to the object store at row/column in this Matrix.
	 *
	 * @param  row specifies the row index in the Matrix
	 * @param  col specifies the column index in the Matrix
	 *
	 * @return a reference to the object at row/column in this Matrix
	*/
  public Object getObject (int row, int col) throws ArrayIndexOutOfBoundsException
  {return values[row][col];}
  
  
  
  
  
	/** 
	 * Returns a 1 x column Matrix storing references to all the objects
	 *   stored in row row in this Matrix.
	 *
	 * @param  row specifies the row index in the Matrix
	 *
	 * @return a 1 x column Matrix storing references to all the objects
	 *   stored in row in this Matrix
	*/
  public Matrix getRow (int row) throws ArrayIndexOutOfBoundsException
  {
    Matrix answer = new Matrix(1,getColSize());
    for (int c=0; c<getColSize(); c++)
      answer.values[0][c] = values [0][c];
    return answer;
  }
  
  
  
  
  
	/** 
	 * Returns a row x 1 Matrix storing references to all the objects
	 *   stored in column col in this Matrix.
	 *
	 * @param  col specifies the column index in the Matrix
	 *
	 * @return a 1 x column Matrix storing references to all the objects
	 *   stored in row in this Matrix
	*/
  public Matrix getCol (int col) throws ArrayIndexOutOfBoundsException
  {
    Matrix answer = new Matrix(getRowSize(),1);
    for (int r=0; r<getRowSize(); r++)
      answer.values[r][0] = values [r][0];
    return answer;
  }
  
  
  
  
  
	/** 
	 * Sets a reference to the object store at row/column in this Matrix.
	 *
	 * @param  row specifies the row index in the Matrix
	 * @param  col specifies the column index in the Matrix
	 *
	*/
  public void setObject (int row, int col, Object o) throws ArrayIndexOutOfBoundsException
  {values[row][col] = o;}
  
  
  
	/** 
	 * Returns a String representation of the state of this dice ensemble.
	 *
	 * @return (1) the number of dice in the ensemble, (2) the number of
	 *         sides on each die, and (3) the pips for each die
	*/
  public String toString ()
  {
    String answer = "Matrix[rows="+getRowSize()+"/"+
          "cols="+getColSize();
    for (int r=0; r<getRowSize(); r++)
      for (int c=0; c<getColSize(); c++)
          answer += "["+r+","+c+"]="+getObject(r,c);
    return answer+"]";
  }
  
  
  
  //No JavaDoc comments for private instance variables
  
  private Object[][] values;  //An array of arrays
}




