//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Interface      : ReverseAComparator
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   A decorator the reverses the result of a Comparator.   
//
// Future Plans   : None
//
// Program History:
//   9/26/04: R. Pattis - Operational for 15-200
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;


import java.util.Comparator;




/** 
 * This class is a decorator that reverses the result of the
 *  <code>Comparator</code> object that it is constructed with.
 * Use it if you have a <code>Comparator</code> class that you are
 *   using, say, to sort an array, but you want the array to be sorted
 *   in the reverse order.
 * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
**/
public class ReverseAComparator<T> implements Comparator<T> {

	/** 
	 * Constructs a <code>ReverseAComparator</code> object, whose
	 *   <code>compare</b> method always returns the oppositve (negative
	 *   value) of its <code>Comparator</code> argument.
	 * Note for equality, -0 = 0.
	 *
	 * @param  c - the <code>Comparator</code> to reverse
	*/
  public ReverseAComparator(Comparator<T> c)
  {realComparator = c;}



  /**
   * Compares its two arguments for order.
   * Returns a negative integer, zero, or a positive integer as the
   *   first argument is less than, equal to, or greater than the second.
   *
   * @param o1 - the first object to be compared
   * @param o2 - the second object to be compared
   *  
   * @return a negative integer, zero, or a positive integer as the
   *   first argument is less than, equal to, or greater than the second
  **/
  public int compare (T o1, T o2)
  {return -realComparator.compare(o1,o2);}




  /**
   * Indicates whether some other object is "equal to" this Comparator. 
   * This method must obey the general contract of Object.equals(Object).
   * Additionally, this method can return true only if the specified
   *   Object is also a comparator and it imposes the same ordering as this
   *   comparator.
   * Thus, comp1.equals(comp2) implies that
   *   sgn(comp1.compare(o1, o2))==sgn(comp2.compare(o1, o2)) for every
   *   object reference o1 and o2.
   *
   * @param obj - the reference object with which to compare.
   *
   * @return true only if the specified object is also a comparator and 
   *   it imposes the same ordering as this comparator.
  **/
  public boolean equals(Object obj)
  {
    return (obj instanceof ReverseAComparator) && 
            realComparator.equals(((ReverseAComparator)obj).realComparator);
  }


  /**
   * Store the original Comparator, whose value is always reversed
  **/
  private final Comparator<T> realComparator;
}